@@ -10,7 +10,8 @@ You will need to execute this command in two cases:
 Credits: Heavily inspired by django-transmeta's sync_transmeta_db command.
 """
 from optparse import make_option
-from django.core.management.base import NoArgsCommand
+import django
+from django.core.management.base import BaseCommand
 from django.core.management.color import no_style
 from django.db import connection, transaction
 from django.utils.six import moves
@@ -45,17 +46,17 @@ def print_missing_langs(missing_langs, field_name, model_name):
         field_name, model_name, ", ".join(missing_langs)))
 
 
-class Command(NoArgsCommand):
+class Command(BaseCommand):
     help = ('Detect new translatable fields or new available languages and'
             ' sync database structure. Does not remove columns of removed'
             ' languages or undeclared fields.')
 
-    option_list = NoArgsCommand.option_list + (
+    option_list = BaseCommand.option_list + (
         make_option('--noinput', action='store_false', dest='interactive', default=True,
                     help='Do NOT prompt the user for input of any kind.'),
     )
 
-    def handle_noargs(self, **options):
+    def handle(self, *args, **options):
         """
         Command execution.
         """
@@ -67,7 +68,11 @@ class Command(NoArgsCommand):
         models = translator.get_registered_models(abstract=False)
         for model in models:
             db_table = model._meta.db_table
-            model_full_name = '%s.%s' % (model._meta.app_label, model._meta.module_name)
+            if django.VERSION < (1, 8):
+                model_name = model._meta.module_name
+            else:
+                model_name = model._meta.model_name
+            model_full_name = '%s.%s' % (model._meta.app_label, model_name)
             opts = translator.get_options_for_model(model)
             for field_name, fields in opts.local_fields.items():
                 # Take `db_column` attribute into account
@@ -88,7 +93,8 @@ class Command(NoArgsCommand):
                     else:
                         print('SQL not executed')
 
-        transaction.commit_unless_managed()
+        if django.VERSION < (1, 6):
+            transaction.commit_unless_managed()
 
         if not found_missing_fields:
             print('No new translatable fields detected')
