@@ -9,14 +9,27 @@ import itertools
 
 from django.db import models
 from django.db.models import FieldDoesNotExist
-from django.db.models.fields.related import RelatedField, RelatedObject
-from django.db.models.sql.where import Constraint
+try:
+    from django.db.models.fields.related import RelatedObject
+    from django.db.models.fields.related import RelatedField
+    NEW_META_API = False
+except ImportError:
+    NEW_META_API = True
+
+try:
+    from django.db.models.query import ValuesQuerySet
+    from django.db.models.sql.where import Constraint
+    NEW_RELATED_API = False
+except ImportError:
+    from django.db.models.query import ValuesIterable
+    NEW_RELATED_API = True  # Django 1.9
+
 from django.utils.six import moves
 from django.utils.tree import Node
 try:
     from django.db.models.lookups import Lookup
     from django.db.models.sql.datastructures import Col
-    NEW_LOOKUPS = True
+    NEW_LOOKUPS = True  # Django 1.7, 1.8
 except ImportError:
     NEW_LOOKUPS = False
 
@@ -117,8 +130,20 @@ _F2TM_CACHE = {}
 
 
 def get_fields_to_translatable_models(model):
-    if model not in _F2TM_CACHE:
-        results = []
+    if model in _F2TM_CACHE:
+        return _F2TM_CACHE[model]
+
+    results = []
+    if NEW_META_API:
+        for f in model._meta.get_fields():
+            if f.is_relation and f.related_model:
+                # The new get_field() will find GenericForeignKey relations.
+                # In that case the 'related_model' attribute is set to None
+                # so it is necessary to check for this value before trying to
+                # get translatable fields.
+                if get_translatable_fields_for_model(f.related_model) is not None:
+                    results.append((f.name, f.related_model))
+    else:
         for field_name in model._meta.get_all_field_names():
             field_object, modelclass, direct, m2m = model._meta.get_field_by_name(field_name)
             # Direct relationship
@@ -129,7 +154,7 @@ def get_fields_to_translatable_models(model):
             if isinstance(field_object, RelatedObject):
                 if get_translatable_fields_for_model(field_object.model) is not None:
                     results.append((field_name, field_object.model))
-        _F2TM_CACHE[model] = dict(results)
+    _F2TM_CACHE[model] = dict(results)
     return _F2TM_CACHE[model]
 
 _C2F_CACHE = {}
@@ -171,16 +196,31 @@ class MultilingualQuerySet(models.query.QuerySet):
                     ordering.append(rewrite_order_lookup_key(self.model, key))
                 self.query.add_ordering(*ordering)
 
+    def __reduce__(self):
+        return multilingual_queryset_factory, (self.__class__.__bases__[0],), self.__getstate__()
+
     # This method was not present in django-linguo
-    def _clone(self, klass=None, *args, **kwargs):
-        if klass is not None and not issubclass(klass, MultilingualQuerySet):
-            class NewClass(klass, MultilingualQuerySet):
-                pass
-            NewClass.__name__ = 'Multilingual%s' % klass.__name__
-            klass = NewClass
-        kwargs.setdefault('_rewrite', self._rewrite)
-        kwargs.setdefault('_populate', self._populate)
-        return super(MultilingualQuerySet, self)._clone(klass, *args, **kwargs)
+    if NEW_RELATED_API:
+        def _clone(self, klass=None, **kwargs):
+            kwargs.setdefault('_rewrite', self._rewrite)
+            kwargs.setdefault('_populate', self._populate)
+            if hasattr(self, 'translation_fields'):
+                kwargs.setdefault('translation_fields', self.translation_fields)
+            if hasattr(self, 'fields_to_del'):
+                kwargs.setdefault('fields_to_del', self.fields_to_del)
+            if hasattr(self, 'original_fields'):
+                kwargs.setdefault('original_fields', self.original_fields)
+            return super(MultilingualQuerySet, self)._clone(**kwargs)
+    else:
+        def _clone(self, klass=None, *args, **kwargs):
+            if klass is not None and not issubclass(klass, MultilingualQuerySet):
+                class NewClass(klass, MultilingualQuerySet):
+                    pass
+                NewClass.__name__ = 'Multilingual%s' % klass.__name__
+                klass = NewClass
+            kwargs.setdefault('_rewrite', self._rewrite)
+            kwargs.setdefault('_populate', self._populate)
+            return super(MultilingualQuerySet, self)._clone(klass, *args, **kwargs)
 
     # This method was not present in django-linguo
     def rewrite(self, mode=True):
@@ -199,7 +239,8 @@ class MultilingualQuerySet(models.query.QuerySet):
         Useful when converting any QuerySet into MultilingualQuerySet.
         """
         self._rewrite_where(self.query.where)
-        self._rewrite_where(self.query.having)
+        if not NEW_RELATED_API:
+            self._rewrite_where(self.query.having)
         self._rewrite_order()
         self._rewrite_select_related()
 
@@ -218,7 +259,7 @@ class MultilingualQuerySet(models.query.QuerySet):
 
     # This method was not present in django-linguo
     def _rewrite_col(self, col):
-        """Django 1.7 column name rewriting"""
+        """Django >= 1.7 column name rewriting"""
         if isinstance(col, Col):
             new_name = rewrite_lookup_key(self.model, col.target.name)
             if col.target.name != new_name:
@@ -255,7 +296,7 @@ class MultilingualQuerySet(models.query.QuerySet):
 
     def _rewrite_select_related(self):
         if isinstance(self.query.select_related, dict):
-            new = []
+            new = {}
             for field_name, value in self.query.select_related.items():
                 new[rewrite_order_lookup_key(self.model, field_name)] = value
             self.query.select_related = new
@@ -279,6 +320,11 @@ class MultilingualQuerySet(models.query.QuerySet):
             return q
         if isinstance(q, Node):
             q.children = list(map(self._rewrite_f, q.children))
+        # Django >= 1.8
+        if hasattr(q, 'lhs'):
+            q.lhs = self._rewrite_f(q.lhs)
+        if hasattr(q, 'rhs'):
+            q.rhs = self._rewrite_f(q.rhs)
         return q
 
     def _filter_or_exclude(self, negate, *args, **kwargs):
@@ -292,7 +338,9 @@ class MultilingualQuerySet(models.query.QuerySet):
         return super(MultilingualQuerySet, self)._filter_or_exclude(negate, *args, **kwargs)
 
     def _get_original_fields(self):
-        return [f.attname for f in self.model._meta.fields if not isinstance(f, TranslationField)]
+        source = (self.model._meta.concrete_fields if hasattr(self.model._meta, 'concrete_fields')
+                  else self.model._meta.fields)
+        return [f.attname for f in source if not isinstance(f, TranslationField)]
 
     def order_by(self, *field_names):
         """
@@ -354,6 +402,16 @@ class MultilingualQuerySet(models.query.QuerySet):
     def raw_values(self, *fields):
         return super(MultilingualQuerySet, self).values(*fields)
 
+    def _values(self, *original, **kwargs):
+        if not kwargs.get('prepare', False):
+            return super(MultilingualQuerySet, self)._values(*original)
+        new_fields, translation_fields = append_fallback(self.model, original)
+        clone = super(MultilingualQuerySet, self)._values(*list(new_fields))
+        clone.original_fields = tuple(original)
+        clone.translation_fields = translation_fields
+        clone.fields_to_del = new_fields - set(original)
+        return clone
+
     # This method was not present in django-linguo
     def values(self, *fields):
         if not self._rewrite:
@@ -361,7 +419,12 @@ class MultilingualQuerySet(models.query.QuerySet):
         if not fields:
             # Emulate original queryset behaviour: get all fields that are not translation fields
             fields = self._get_original_fields()
-        return self._clone(klass=FallbackValuesQuerySet, setup=True, _fields=fields)
+        if NEW_RELATED_API:
+            clone = self._values(*fields, prepare=True)
+            clone._iterable_class = FallbackValuesIterable
+            return clone
+        else:
+            return self._clone(klass=FallbackValuesQuerySet, setup=True, _fields=fields)
 
     # This method was not present in django-linguo
     def values_list(self, *fields, **kwargs):
@@ -376,7 +439,14 @@ class MultilingualQuerySet(models.query.QuerySet):
         if not fields:
             # Emulate original queryset behaviour: get all fields that are not translation fields
             fields = self._get_original_fields()
-        return self._clone(klass=FallbackValuesListQuerySet, setup=True, flat=flat, _fields=fields)
+        if NEW_RELATED_API:
+            clone = self._values(*fields, prepare=True)
+            clone._iterable_class = (FallbackFlatValuesListIterable if flat
+                                     else FallbackValuesListIterable)
+            return clone
+        else:
+            return self._clone(klass=FallbackValuesListQuerySet, setup=True, flat=flat,
+                               _fields=fields)
 
     # This method was not present in django-linguo
     def dates(self, field_name, *args, **kwargs):
@@ -386,56 +456,91 @@ class MultilingualQuerySet(models.query.QuerySet):
         return super(MultilingualQuerySet, self).dates(new_key, *args, **kwargs)
 
 
-class FallbackValuesQuerySet(models.query.ValuesQuerySet, MultilingualQuerySet):
-    def _setup_query(self):
-        original = self._fields
-        new_fields, self.translation_fields = append_fallback(self.model, original)
-        self._fields = list(new_fields)
-        self.fields_to_del = new_fields - set(original)
-        super(FallbackValuesQuerySet, self)._setup_query()
+if NEW_RELATED_API:
+    class FallbackValuesIterable(ValuesIterable):
+        class X(object):
+            # This stupid class is needed as object use __slots__ and has no __dict__.
+            pass
 
-    class X(object):
-        # This stupid class is needed as object use __slots__ and has no __dict__.
-        pass
+        def __iter__(self):
+            instance = self.X()
+            for row in super(FallbackValuesIterable, self).__iter__():
+                instance.__dict__.update(row)
+                for key in self.queryset.translation_fields:
+                    row[key] = getattr(self.queryset.model, key).__get__(instance, None)
+                for key in self.queryset.fields_to_del:
+                    del row[key]
+                yield row
 
-    def iterator(self):
-        instance = self.X()
-        for row in super(FallbackValuesQuerySet, self).iterator():
-            instance.__dict__.update(row)
-            for key in self.translation_fields:
-                row[key] = getattr(self.model, key).__get__(instance, None)
-            for key in self.fields_to_del:
-                del row[key]
-            yield row
+    class FallbackValuesListIterable(FallbackValuesIterable):
+        def __iter__(self):
+            fields = self.queryset.original_fields
+            fields += tuple(f for f in self.queryset.query.annotation_select if f not in fields)
+            for row in super(FallbackValuesListIterable, self).__iter__():
+                yield tuple(row[f] for f in fields)
 
-    def _clone(self, klass=None, setup=False, **kwargs):
-        c = super(FallbackValuesQuerySet, self)._clone(klass, **kwargs)
-        c.fields_to_del = self.fields_to_del
-        c.translation_fields = self.translation_fields
-        if setup and hasattr(c, '_setup_query'):
-            c._setup_query()
-        return c
+    class FallbackFlatValuesListIterable(FallbackValuesListIterable):
+        def __iter__(self):
+            for row in super(FallbackFlatValuesListIterable, self).__iter__():
+                yield row[0]
 
+else:
+    class FallbackValuesQuerySet(ValuesQuerySet, MultilingualQuerySet):
+        def _setup_query(self):
+            original = self._fields
+            new_fields, self.translation_fields = append_fallback(self.model, original)
+            self._fields = list(new_fields)
+            self.fields_to_del = new_fields - set(original)
+            super(FallbackValuesQuerySet, self)._setup_query()
 
-class FallbackValuesListQuerySet(FallbackValuesQuerySet):
-    def iterator(self):
-        for row in super(FallbackValuesListQuerySet, self).iterator():
-            if self.flat and len(self.original_fields) == 1:
-                yield row[self.original_fields[0]]
-            else:
-                yield tuple(row[f] for f in self.original_fields)
+        class X(object):
+            # This stupid class is needed as object use __slots__ and has no __dict__.
+            pass
 
-    def _setup_query(self):
-        self.original_fields = self._fields
-        super(FallbackValuesListQuerySet, self)._setup_query()
+        def iterator(self):
+            instance = self.X()
+            for row in super(FallbackValuesQuerySet, self).iterator():
+                instance.__dict__.update(row)
+                for key in self.translation_fields:
+                    row[key] = getattr(self.model, key).__get__(instance, None)
+                for key in self.fields_to_del:
+                    del row[key]
+                yield row
 
-    def _clone(self, *args, **kwargs):
-        clone = super(FallbackValuesListQuerySet, self)._clone(*args, **kwargs)
-        clone.original_fields = self.original_fields
-        if not hasattr(clone, "flat"):
-            # Only assign flat if the clone didn't already get it from kwargs
-            clone.flat = self.flat
-        return clone
+        def _clone(self, klass=None, setup=False, **kwargs):
+            c = super(FallbackValuesQuerySet, self)._clone(klass, **kwargs)
+            c.fields_to_del = self.fields_to_del
+            c.translation_fields = self.translation_fields
+            if setup and hasattr(c, '_setup_query'):
+                c._setup_query()
+            return c
+
+    class FallbackValuesListQuerySet(FallbackValuesQuerySet):
+        def iterator(self):
+            fields = self.original_fields
+            if hasattr(self, 'aggregate_names'):
+                # Django <1.8
+                fields += tuple(f for f in self.aggregate_names if f not in fields)
+            if hasattr(self, 'annotation_names'):
+                # Django >=1.8
+                fields += tuple(f for f in self.annotation_names if f not in fields)
+            for row in super(FallbackValuesListQuerySet, self).iterator():
+                if self.flat and len(fields) == 1:
+                    yield row[fields[0]]
+                else:
+                    yield tuple(row[f] for f in fields)
+
+        def _setup_query(self):
+            self.original_fields = tuple(self._fields)
+            super(FallbackValuesListQuerySet, self)._setup_query()
+
+        def _clone(self, *args, **kwargs):
+            clone = super(FallbackValuesListQuerySet, self)._clone(*args, **kwargs)
+            clone.original_fields = self.original_fields
+            if not hasattr(clone, "flat"):
+                # Only assign flat if the clone didn't already get it from kwargs
+                clone.flat = self.flat
+            return clone
 
 
 def get_queryset(obj):
@@ -445,6 +550,16 @@ def get_queryset(obj):
         return obj.get_query_set()
 
 
+def multilingual_queryset_factory(old_cls, instantiate=True):
+    if old_cls == models.query.QuerySet:
+        NewClass = MultilingualQuerySet
+    else:
+        class NewClass(old_cls, MultilingualQuerySet):
+            pass
+        NewClass.__name__ = 'Multilingual%s' % old_cls.__name__
+    return NewClass() if instantiate else NewClass
+
+
 class MultilingualQuerysetManager(models.Manager):
     """
     This class gets hooked in MRO just before plain Manager, so that every call to
@@ -455,13 +570,7 @@ class MultilingualQuerysetManager(models.Manager):
         return self._patch_queryset(qs)
 
     def _patch_queryset(self, qs):
-        if qs.__class__ == models.query.QuerySet:
-            qs.__class__ = MultilingualQuerySet
-        else:
-            class NewClass(qs.__class__, MultilingualQuerySet):
-                pass
-            NewClass.__name__ = 'Multilingual%s' % qs.__class__.__name__
-            qs.__class__ = NewClass
+        qs.__class__ = multilingual_queryset_factory(qs.__class__, instantiate=False)
         qs._post_init()
         qs._rewrite_applied_operations()
         return qs
