@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+from django import VERSION
 from django.utils.six import with_metaclass
 from django.core.exceptions import ImproperlyConfigured
 from django.db.models import Manager, ForeignKey, OneToOneField
@@ -14,6 +15,9 @@ from modeltranslation.manager import (MultilingualManager, MultilingualQuerysetM
 from modeltranslation.utils import build_localized_fieldname, parse_field
 
 
+NEW_RELATED_API = VERSION >= (1, 9)
+
+
 class AlreadyRegistered(Exception):
     pass
 
@@ -150,7 +154,12 @@ def add_translation_fields(model, opts):
     # Rebuild information about parents fields. If there are opts.local_fields, field cache would be
     # invalidated (by model._meta.add_field() function). Otherwise, we need to do it manually.
     if len(opts.local_fields) == 0:
-        model._meta._fill_fields_cache()
+        try:
+            model._meta._fill_fields_cache()
+        except AttributeError:
+            # Django 1.8 removed _fill_fields_cache
+            model._meta._expire_cache()
+            model._meta.get_fields()
 
 
 def has_custom_queryset(manager):
@@ -180,6 +189,18 @@ def add_manager(model):
                                          MultilingualQuerysetManager):
                 use_for_related_fields = getattr(
                     manager.__class__, "use_for_related_fields", not has_custom_queryset(manager))
+                _old_module = manager.__module__
+                _old_class = manager.__class__.__name__
+
+                def deconstruct(self):
+                    return (
+                        False,  # as_manager
+                        '%s.%s' % (self._old_module, self._old_class),  # manager_class
+                        None,  # qs_class
+                        self._constructor_args[0],  # args
+                        self._constructor_args[1],  # kwargs
+                    )
+
             manager.__class__ = NewMultilingualManager
 
     for _, attname, cls in model._meta.concrete_managers + model._meta.abstract_managers:
@@ -242,6 +263,22 @@ def patch_clean_fields(model):
     model.clean_fields = new_clean_fields
 
 
+def patch_get_deferred_fields(model):
+    """
+    Django >= 1.8: patch detecting deferred fields. Crucial for only/defer to work.
+    """
+    if not hasattr(model, 'get_deferred_fields'):
+        return
+    old_get_deferred_fields = model.get_deferred_fields
+
+    def new_get_deferred_fields(self):
+        sup = old_get_deferred_fields(self)
+        if hasattr(self, '_fields_were_deferred'):
+            sup.update(self._fields_were_deferred)
+        return sup
+    model.get_deferred_fields = new_get_deferred_fields
+
+
 def patch_metaclass(model):
     """
     Monkey patches original model metaclass to exclude translated fields on deferred subclasses.
@@ -260,8 +297,13 @@ def patch_metaclass(model):
         def __new__(cls, name, bases, attrs):
             if attrs.get('_deferred', False):
                 opts = translator.get_options_for_model(model)
+                were_deferred = set()
                 for field_name in opts.fields.keys():
-                    attrs.pop(field_name, None)
+                    if attrs.pop(field_name, None):
+                        # Field was deferred. Store this for future reference.
+                        were_deferred.add(field_name)
+                if len(were_deferred):
+                    attrs['_fields_were_deferred'] = were_deferred
             return super(translation_deferred_mcs, cls).__new__(cls, name, bases, attrs)
     # Assign to __metaclass__ wouldn't work, since metaclass search algorithm check for __class__.
     # http://docs.python.org/2/reference/datamodel.html#__metaclass__
@@ -278,6 +320,9 @@ def delete_cache_fields(model):
         except AttributeError:
             pass
 
+    if hasattr(model._meta, '_expire_cache'):
+        model._meta._expire_cache()
+
 
 def populate_translation_fields(sender, kwargs):
     """
@@ -395,7 +440,10 @@ class Translator(object):
                 add_translation_fields(model, opts)
 
             # Delete all fields cache for related model (parent and children)
-            for related_obj in model._meta.get_all_related_objects():
+            related = ((f for f in model._meta.get_fields() if (f.one_to_many or f.one_to_one)
+                        and f.auto_created) if NEW_RELATED_API
+                       else model._meta.get_all_related_objects())
+            for related_obj in related:
                 delete_cache_fields(related_obj.model)
 
             # Set MultilingualManager
@@ -410,8 +458,9 @@ class Translator(object):
             # Patch clean_fields to verify form field clearing
             patch_clean_fields(model)
 
-            # Patch __metaclass__ to allow deferring to work
+            # Patch __metaclass__ and other methods to allow deferring to work
             patch_metaclass(model)
+            patch_get_deferred_fields(model)
 
             # Substitute original field with descriptor
             model_fallback_languages = getattr(opts, 'fallback_languages', None)
@@ -435,7 +484,13 @@ class Translator(object):
                     setattr(model, field.get_attname(), desc)
 
                     # Set related field names on other model
-                    if not field.rel.is_hidden():
+                    if NEW_RELATED_API and not field.remote_field.is_hidden():
+                        other_opts = self._get_options_for_model(field.remote_field.to)
+                        other_opts.related = True
+                        other_opts.related_fields.append(field.related_query_name())
+                        # Add manager in case of non-registered model
+                        add_manager(field.remote_field.to)
+                    elif not NEW_RELATED_API and not field.rel.is_hidden():
                         other_opts = self._get_options_for_model(field.rel.to)
                         other_opts.related = True
                         other_opts.related_fields.append(field.related_query_name())
@@ -443,7 +498,10 @@ class Translator(object):
 
                 if isinstance(field, OneToOneField):
                     # Fix translated_field caching for SingleRelatedObjectDescriptor
-                    sro_descriptor = getattr(field.rel.to, field.related.get_accessor_name())
+                    sro_descriptor = (
+                        getattr(field.remote_field.to, field.remote_field.get_accessor_name())
+                        if NEW_RELATED_API
+                        else getattr(field.rel.to, field.related.get_accessor_name()))
                     patch_related_object_descriptor_caching(sro_descriptor)
 
     def unregister(self, model_or_iterable):
@@ -522,3 +580,7 @@ class Translator(object):
 
 # This global object represents the singleton translator object
 translator = Translator()
+
+
+# Re-export the decorator for convenience
+from modeltranslation.decorators import register  # NOQA re-export
